#  openMat for blender - A demo to show how to use the 
#  import and export library 
#  for the openMat database format
#
#  Copyright (C) 2012 Stefan Drechsler
# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####


import bpy.utils
import bpy.types 
import bpy.path
from bpy.props import EnumProperty

from bpy_extras.image_utils import load_image
from bpy_extras.object_utils import object_data_add
from bpy_extras.object_utils import AddObjectHelper
from bpy_extras.io_utils import ImportHelper

import os
from openMat import MaterialDatabase

import acousticAddons.MaterialInterpolated

EXT_LIST = {
    'xml': ('xml', ),
    'oma': ('oma', ),
    }

EXTENSIONS = [ext for ext_ls in EXT_LIST.values() for ext in ext_ls]

def calcPlace(i):
    zoff = i // 2880
    i1 = i% 2880
    layer = i1 // 144
    layer = 19 - layer
    i2 = i1 % 144
    zeile = i2 // 48
    i3 = i2 % 48
    spalte = i3 // 16
    i4 = i3 % 16
    zzeile = i4 // 4
    sspalte = i4 % 4
    xmin = -7.0 + 7.0 * spalte + 1.5 * sspalte
    xmax = xmin + 1.0
    ymin = -7.0 + 7.0 * zeile + 1.5 * zzeile
    ymax = ymin + 1.0
    return (xmin, xmax, ymin, ymax, zoff, layer)

def create_image_plane(self, context, material, i, amaterial):

    (xmin, xmax, ymin, ymax, zoff, layer) = calcPlace(i)


    verts = ((xmin, ymin, zoff),
             (xmin, ymax, zoff),
             (xmax, ymax, zoff),
             (xmax, ymin, zoff),
             )
    faces = ((0, 3, 2, 1), )

    matName = amaterial.getNames()[0].getContent()

    mesh_data = bpy.data.meshes.new(matName)
    mesh_data.from_pydata(verts, [], faces)
    mesh_data.update()
    
    object_data_add(context, mesh_data, operator=self) 
    plane = context.scene.objects.active
    plane.data.uv_textures.new()
    plane.data.materials.append(material)
    
    if material.texture_slots[0]:
        img = material.texture_slots[0].texture.image
        plane.data.uv_textures[0].data[0].image = img
    

    material.game_settings.use_backface_culling = False
    material.game_settings.alpha_blend = 'ALPHA'
    return plane, layer

def moveToLayer(plane, layer):
    layerList = [False, False, False, False, False, 
                 False, False, False, False, False, 
                 False, False, False, False, False, 
                 False, False, False, False, False]
    layerList[layer] = True
    layerTuple = tuple(layerList)
    plane.select = True
    bpy.ops.object.move_to_layer(layers=layerTuple)
    plane.select = False

def is_image_fn_any(fn):
    ext = os.path.splitext(fn)[1].lstrip(".").lower()
    return ext in EXTENSIONS

def is_image_fn_single(fn, ext_key):
    ext = os.path.splitext(fn)[1].lstrip(".").lower()
    return ext in EXT_LIST[ext_key]

def generate_paths(self):
    directory, fn = os.path.split(self.filepath)
    if fn:
        # test for extension
        if not is_image_fn_any(fn):
            return [], directory
        return [self.filepath], directory

    if not fn:
        imagepaths = []
        files_in_directory = os.listdir(directory)
        # clean files from nonimages
        files_in_directory = [fn for fn in files_in_directory
                              if is_image_fn_any(fn)]
        # clean from unwanted extensions
        if self.extension != "*":
            files_in_directory = [fn for fn in files_in_directory
                                  if is_image_fn_single(fn, self.extension)]
        # create paths
        for fn in files_in_directory:
            imagepaths.append(os.path.join(directory, fn))

        #print(imagepaths)
        return imagepaths, directory

def load_materials(path, directory):
    amdb = MaterialDatabase.MaterialDatabase()
    amdb.readXML(path)
    retVal = []
    for i in range(amdb.length()):
        retVal.append(amdb.getItem(i))
    return retVal

def create_image_textures(self, image, colorkey):
    fn_full = os.path.normpath(bpy.path.abspath(image.filepath))
    # look for texture with importsettings
    for texture in bpy.data.textures:
        if texture.type == 'IMAGE':
            tex_img = texture.image
            if (tex_img is not None) and (tex_img.library is None):
                fn_tex_full = os.path.normpath(bpy.path.abspath(tex_img.filepath))
                if fn_full == fn_tex_full:
                    texture.use_alpha = False
                    return texture

    # if no texture is found: create one
    texture = bpy.data.textures.new(name=colorkey, type='IMAGE')
    texture.image = image
    texture.use_alpha = False
    
    return texture


def add_texture(texture, material):
    slot = material.texture_slots.add()
    slot.texture = texture
    slot.texture_coords = 'UV'
    slot.scale = (30.0, 30.0, 30.0)
    slot.use_map_alpha = False

def create_material(self, amaterial):
    name_compat = amaterial.getNames()[0].getContent()
    material = bpy.data.materials.new(name=name_compat)
    material.alpha = 1.0
    material.specular_alpha = 1.0
    material.use_transparency = False
    material.use_shadeless = False
    material.use_transparent_shadows = True
    
    alpha = acousticAddons.MaterialInterpolated.getAlpha(amaterial)
    if alpha == None:
        return
    material.acoustic_alpha = alpha
    
    sigma = acousticAddons.MaterialInterpolated.getSigma(amaterial)
    if sigma != None:
        material.acoustic_scatter = sigma
    
    r = 1.0 - 0.5 * (material.acoustic_alpha[0] + material.acoustic_alpha[1])
    g = 1.0 - 0.5 * (material.acoustic_alpha[2] + material.acoustic_alpha[3])
    b = 1.0 - 0.5 * (material.acoustic_alpha[4] + material.acoustic_alpha[5])
    material.diffuse_color = (r, g, b)
    material.game_settings.use_backface_culling = True
    return material

def import_openMat(self, context):
    import_list, directory = generate_paths(self)
    amaterials = []
    textures = []
    planes = []

    for path in import_list:
        amaterials.extend(load_materials(path, directory))

    for i, amaterial in enumerate(amaterials):
        material = create_material(self, amaterial)
        if material == None:
            continue
        textures = amaterial.getTextures()
        for textureitem in textures:
            filename = textureitem.getContent()
            colorkey = textureitem.getColor()
            if filename != '':
                fn = os.path.join(directory, filename) 
                image = load_image(fn)
                if image == None:
                    continue
                image.use_premultiply = False
                texture = create_image_textures(self, image, colorkey)
                add_texture(texture, material)
            
        
        descs = amaterial.getDescriptions()
        if len(descs) > 0:
            material.acoustic_description = descs[0].getContent()
        else:
            material.acoustic_description = "no description available"
            
            
        plane, layer = create_image_plane(self, context, material, i, amaterial)
        moveToLayer(plane, layer)
        planes.append(plane)
        material.game_settings.use_backface_culling = True
        


    context.scene.update()

    self.report({'INFO'}, "Added %i acoustic materials(s)" % len(planes))


class IMPORT_OT_openMat(bpy.types.Operator, ImportHelper, AddObjectHelper):
    """Create mesh plane(s) from openMat """

    bl_idname = "import.open_mat"
    bl_label = "Import acoustic materials"
    bl_options = {'REGISTER', 'UNDO'}

    # -------
    # Options
    extension = EnumProperty(
            name="Extension",
            description="Only import files of this type",
            items=(
            ('*', 'All file formats',
                'Select from all file formats.'),
            ('xml', 'XML (.xml)',
                'xml-ending'),
            ('oma', 'OMA (.oma)', 'openMat')),
            )

    def draw(self, context):
        layout = self.layout

        box = layout.box()
        box.label("Import Options:", icon='FILTER')
        box.prop(self, "extension", icon='FILE_IMAGE')

    def execute(self, context):

        if not bpy.data.is_saved:
            self.relative = False

        # the add utils don't work in this case
        # because many objects are added
        # disable relevant things beforehand
        editmode = context.user_preferences.edit.use_enter_edit_mode
        context.user_preferences.edit.use_enter_edit_mode = False
        if context.active_object\
        and context.active_object.mode == 'EDIT':
            bpy.ops.object.mode_set(mode='OBJECT')

        import_openMat(self, context)

        context.user_preferences.edit.use_enter_edit_mode = editmode
        return {'FINISHED'}

