#  openMat for blender - A demo to show how to use the 
#  import and export library 
#  for the openMat database format
#
#  Copyright (C) 2012 Stefan Drechsler
# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

"""
This tool introduces the calculation of Reverberation times according to 
Sabine and Eyring in Blender.
"""




import bpy
import bpy.types
import bpy.props
import mathutils

import math



bpy.types.Object.surface = bpy.props.FloatProperty(default = 0.0, min=0.0)
bpy.types.Object.volume = bpy.props.FloatProperty(default = 0.0, min=0.0)
bpy.types.Object.tSab = bpy.props.FloatVectorProperty(
    size=6, default = (0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
    min = 0.0, unit = 'TIME')
bpy.types.Object.tEyr = bpy.props.FloatVectorProperty(
    size=6, default = (0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
    min = 0.0, unit = 'TIME')

class SabineOperator(bpy.types.Operator):
    '''Calculates the reverberation time according to Sabine for the selected Object'''
    bl_idname = "object.sabine_operator"
    bl_label = "Sabine Object Operator"

    def calcTsabine(self, context):
        """Calculate the reverberation time with Sabine's formula
           Arguments:
                @context (?): the current context
        """
        obj = context.active_object

        #bpy.ops.object.mode_set(mode='EDIT')
        #bpy.ops.mesh.select_all(action='SELECT')
        #bpy.ops.mesh.normals_make_consistent(inside=True)
        #bpy.ops.object.mode_set(mode='OBJECT')

        scene = context.scene
        obj_mat = obj.matrix_world
        mesh = obj.to_mesh(scene, True, 'PREVIEW')
        mesh.transform(obj_mat)
        
        areaGeo = 0.0
        areaAbs = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        vol = 0.0
        volmatrix = mathutils.Matrix()
        verts = mesh.vertices[:]
        materials = mesh.materials[:]
        for fac in mesh.polygons:
            facarea = fac.area
            areaGeo = areaGeo + facarea
            imat = fac.material_index
            mat = materials[imat]
            for i in range(len(areaAbs)):
                areaAbs[i] = areaAbs[i] + mat.acoustic_alpha[i] * facarea
            v0 = mathutils.Vector(verts[fac.vertices[0]].co)
            for i in range(2, len(fac.vertices)):
                v1 = mathutils.Vector(verts[fac.vertices[i-1]].co)
                v2 = mathutils.Vector(verts[fac.vertices[i  ]].co)
                volmatrix[0][0:3] = v0
                volmatrix[1][0:3] = v1
                volmatrix[2][0:3] = v2
                v012 = volmatrix.determinant()
                vol = vol + v012
                
        obj.surface = areaGeo
        vol = - vol
        obj.volume = vol / 6.0
        for i in range(len(areaAbs)):
            m = scene.acoustic_air_damping[i]
            V = obj.volume
            A = areaAbs[i]
            S = obj.surface
            c = scene.audio_doppler_speed
            k161 = 24.0 * math.log(10.0) / c
            obj.tSab[i] = k161 * V / (A + 4.0 * m * V)
            obj.tEyr[i] = - k161 * V / (S * math.log(1.0 - A / S) - 4.0 * m * V)
        return

    @classmethod
    def poll(cls, context):
        obj = context.active_object
        return (obj and obj.type == 'MESH')

    def execute(self, context):
        self.calcTsabine(context)
        return {'FINISHED'}


class AcousticObjectPanel(bpy.types.Panel):
    bl_label = "Acoustics"
    bl_idname = "Object_PT_acoustics"
    bl_space_type = "PROPERTIES"
    bl_region_type = "WINDOW"
    bl_context = "object"

    def draw(self, context):
        layout = self.layout

        obj = context.object

        row = layout.row()
        row.label(text="Active object is: " + obj.name)
        row = layout.row()
        row.operator("object.sabine_operator", "get T_Sabine")
        row = layout.row()
        nstr = "S = {0:.3f}".format(obj.surface)
        row.label(text=nstr)
        nstr = " V = {0:.3f}".format(obj.volume)
        row.label(text=nstr)
        row = layout.row()
        row.label(text=" ")
        row.label(text="T_sabine ")
        row.label(text="T_Eyring ")

        row = layout.row()
        row.label(text="125 Hz ")
        nstr = "{0:.3f}".format(obj.tSab[0])
        row.label(text=nstr)
        nstr = "{0:.3f}".format(obj.tEyr[0])
        row.label(text=nstr)

        row = layout.row()
        row.label(text="250 Hz ")
        nstr = "{0:.3f}".format(obj.tSab[1])
        row.label(text=nstr)
        nstr = "{0:.3f}".format(obj.tEyr[1])
        row.label(text=nstr)

        row = layout.row()
        row.label(text="500 Hz ")
        nstr = "{0:.3f}".format(obj.tSab[2])
        row.label(text=nstr)
        nstr = "{0:.3f}".format(obj.tEyr[2])
        row.label(text=nstr)

        row = layout.row()
        row.label(text="1 kHz ")
        nstr = "{0:.3f}".format(obj.tSab[3])
        row.label(text=nstr)
        nstr = "{0:.3f}".format(obj.tEyr[3])
        row.label(text=nstr)

        row = layout.row()
        row.label(text="2 kHz ")
        nstr = "{0:.3f}".format(obj.tSab[4])
        row.label(text=nstr)
        nstr = "{0:.3f}".format(obj.tEyr[4])
        row.label(text=nstr)

        row = layout.row()
        row.label(text="4 kHz ")
        nstr = "{0:.3f}".format(obj.tSab[5])
        row.label(text=nstr)
        nstr = "{0:.3f}".format(obj.tEyr[5])
        row.label(text=nstr)
